// The point and size class used in this program
function Point(x, y) {
    this.x = (x)? parseFloat(x) : 0.0;
    this.y = (y)? parseFloat(y) : 0.0;
}

function Size(w, h) {
    this.w = (w)? parseFloat(w) : 0.0;
    this.h = (h)? parseFloat(h) : 0.0;
}

// Helper function for checking intersection between two rectangles
function intersect(pos1, size1, pos2, size2) {
    return (pos1.x < pos2.x + size2.w && pos1.x + size1.w > pos2.x &&
            pos1.y < pos2.y + size2.h && pos1.y + size1.h > pos2.y);
}


// The player class used in this program
function Player() {
    this.node = svgdoc.getElementById("player");
	var player = this.node;
    this.position = PLAYER_INIT_POS;
    this.motion = motionType.NONE;
    this.verticalSpeed = 0;
	this.face = motionType.RIGHT;
	this.needFlip = false;
}

Player.prototype.isOnPlatform = function() {
    var platforms = svgdoc.getElementById("platforms");
    for (var i = 0; i < platforms.childNodes.length; i++) {
        var node = platforms.childNodes.item(i);
        if (node.nodeName != "rect") continue;

        var x = parseFloat(node.getAttribute("x"));
        var y = parseFloat(node.getAttribute("y"));
        var w = parseFloat(node.getAttribute("width"));
        var h = parseFloat(node.getAttribute("height"));

        if (((this.position.x + PLAYER_SIZE.w > x && this.position.x < x + w) ||
             ((this.position.x + PLAYER_SIZE.w) == x && this.motion == motionType.RIGHT) ||
             (this.position.x == (x + w) && this.motion == motionType.LEFT)) &&
            this.position.y + PLAYER_SIZE.h == y) return true;
    }
    if (this.position.y + PLAYER_SIZE.h == SCREEN_SIZE.h) return true;

    return false;
}

Player.prototype.collidePlatform = function(position) {
    var platforms = svgdoc.getElementById("platforms");
    for (var i = 0; i < platforms.childNodes.length; i++) {
        var node = platforms.childNodes.item(i);
        if (node.nodeName != "rect") continue;

        var x = parseFloat(node.getAttribute("x"));
        var y = parseFloat(node.getAttribute("y"));
        var w = parseFloat(node.getAttribute("width"));
        var h = parseFloat(node.getAttribute("height"));
        var pos = new Point(x, y);
        var size = new Size(w, h);

        if (intersect(position, PLAYER_SIZE, pos, size)) {
            position.x = this.position.x;
            if (intersect(position, PLAYER_SIZE, pos, size)) {
                if (this.position.y >= y + h)
                    position.y = y + h;
                else
                    position.y = y - PLAYER_SIZE.h;
                this.verticalSpeed = 0;
            }
        }
    }
}

Player.prototype.collideScreen = function(position) {
    if (position.x < 0) position.x = 0;
    if (position.x + PLAYER_SIZE.w > SCREEN_SIZE.w) position.x = SCREEN_SIZE.w - PLAYER_SIZE.w;
    if (position.y < 0) {
        position.y = 0;
        this.verticalSpeed = 0;
    }
    if (position.y + PLAYER_SIZE.h > SCREEN_SIZE.h) {
        position.y = SCREEN_SIZE.h - PLAYER_SIZE.h;
        this.verticalSpeed = 0;
    }
}
var MONSTER_TYPE = {NORMAL:0, SHOOTER:1};
// The monster class used in this program
function Monster(node,x,y,destination,monsterType) {
    this.node = node
    this.position = new Point(x, y);
    this.motion = motionType.NONE;
  	this.face = motionType.RIGHT;
  	this.needFlip = false;
    this.destination = destination;
    this.type = monsterType;
}

//
// Below are constants used in the game
//
var PLAYER_SIZE = new Size(40, 40);         // The size of the player
var SCREEN_SIZE = new Size(600, 560);       // The size of the game screen
var PLAYER_INIT_POS  = new Point(0, 420);   // The initial position of the player

var MOVE_DISPLACEMENT = 5;                  // The speed of the player in motion
var JUMP_SPEED = 15;                        // The speed of the player jumping
var VERTICAL_DISPLACEMENT = 1;              // The displacement of vertical speed

var GAME_INTERVAL = 25;                     // The time interval of running the game

var BULLET_SIZE = new Size(10, 10);         // The speed of a bullet
var BULLET_SPEED = 10.0;                    // The speed of a bullet
                                            //  = pixels it moves each game loop
var SHOOT_INTERVAL = 200.0;                 // The period when shooting is disabled
var canShoot = true;                        // A flag indicating whether the player can shoot a bullet

var MONSTER_SIZE = new Size(40, 40);
var MONSTER_VALUE = 100;
var MONSTER_NUM = 7;
var MONSTER_MIN_DISTANCE = 100;
var MONSTER_SPEED = 1;
var monster_canShoot = true;
var MONSTER_SHOOT_INTERVAL = 3000.0;

var GOOD_THINGS_NUM = 9;
var GOOD_THINGS_VALUE = 100;
var GOOD_THINGS_SIZE = new Size(40, 40);

var TIME_TO_WIN = 140;
var TIME_INTERVAL = 1000;

var PORTAL_SIZE = new Size(48, 49);
//
// Variables in the game
//
var motionType = {NONE:0, LEFT:1, RIGHT:2}; // Motion enum

var svgdoc = null;                          // SVG root document node
var player = null;                          // The player object
var gameInterval = null;                    // The interval
var zoom = 1.0;                             // The zoom level of the screen
var score = 0;                              // The score of the game
var playerName = null;
var monsters = {};
var bulletNum = 8;
var timeInterval = null;
var rTime = TIME_TO_WIN;
var win = false;
var dpInterval = null;
var cheatMode = false;
//
// The load function for the SVG document
//
function load(evt) {
    // Set the root node to the global variable
    svgdoc = evt.target.ownerDocument;
	playerName = prompt("Hero, what is your name?")
	if (playerName == ""||playerName == null) playerName="Anonymous";

	//start();
}

function startTimer(){
	if (rTime > 0){
		svgdoc.getElementById("time").firstChild.data = --rTime;
		svgdoc.getElementById("timebar").setAttribute("width",rTime);
		if (rTime <= 0){
			stopTimer();
			gameOver();
		}
	} else{
		gameOver();
	}
}

function stopTimer(){
	clearInterval(timeInterval);
}

function start(zoom){
  	if(zoom==true){
		setZoom();
	}
    svgdoc.getElementById("startScreen").setAttribute("visibility","hidden");
  	svgdoc.getElementById("highscoretable").setAttribute("visibility","hidden");
	// Attach keyboard events
    svgdoc.documentElement.addEventListener("keydown", keydown, false);
    svgdoc.documentElement.addEventListener("keyup", keyup, false);

    // Remove text nodes in the 'platforms' group
    cleanUpGroup("platforms", true);
    cleanUpGroup("monsters", false);
    cleanUpGroup("bullets", false);
    cleanUpGroup("goodThings", false);
    cleanUpGroup("highscoretext", false);
    bulletNum = 8;
    score = 0;
	win = false;
	
    svgdoc.getElementById("score").firstChild.data = score;
    // Create the player
    player = new Player();
  	svgdoc.getElementById("playerName").textContent = playerName;

      // Create the monsters
  	for(var i=0;i<MONSTER_NUM;i++){
  		x = randomXY().x;
  		y = randomXY().y;
  		if( Math.abs(PLAYER_INIT_POS.x - x) < MONSTER_MIN_DISTANCE && Math.abs(PLAYER_INIT_POS.y - y) < MONSTER_MIN_DISTANCE){
  			i--;
  			continue;
  		}
      if(i==MONSTER_NUM-1){
        createMonster(i, x, y, MONSTER_TYPE.SHOOTER);
      }else {
        createMonster(i, x, y, MONSTER_TYPE.NORMAL);
      }
  	}

    // Create good things
    for(var i=0;i<GOOD_THINGS_NUM;i++){
		x = randomXY().x;
		y = randomXY().y;
		// check within platform
		var platforms = svgdoc.getElementById("platforms").getElementsByTagName("rect");
		var inter = false;
		for(platform in platforms){
			var platformNode = platforms[platform];
			if (typeof(platformNode)!="object"){
			  continue;
			}
			var px = parseInt(platformNode.getAttribute("x"));
			var py = parseInt(platformNode.getAttribute("y"));
			var pw = parseInt(platformNode.getAttribute("width"));
			var ph = parseInt(platformNode.getAttribute("height"));

			if (intersect(new Point(px, py), new Size(pw,ph), new Point(x,y), GOOD_THINGS_SIZE)) {
			  inter = true;
			}
		}
		var portal1 = svgdoc.getElementById("portal1");
		var portal2 = svgdoc.getElementById("portal2");
		var px1 = parseInt(portal1.getAttribute("x"));
		var py1 = parseInt(portal1.getAttribute("y"));
		var px2 = parseInt(portal2.getAttribute("x"));
		var py2 = parseInt(portal2.getAttribute("y"));	
		if (intersect(new Point(px1, py1), new Size(PORTAL_SIZE.w,PORTAL_SIZE.h), new Point(x,y), GOOD_THINGS_SIZE)||intersect(new Point(px2, py2), new Size(PORTAL_SIZE.w,PORTAL_SIZE.h), new Point(x,y), GOOD_THINGS_SIZE)) {
          inter = true;
        }
		if (inter){
			i--;
			continue;
		}
		createGoodThing(x,y);
    }

    createDisappearingPlatforms();
  	// reset and start the remaining time count down
  	rTime = TIME_TO_WIN;
  	svgdoc.getElementById("time").firstChild.data = rTime;
  	svgdoc.getElementById("timebar").setAttribute("width",rTime);
  	timeInterval = setInterval("startTimer()", TIME_INTERVAL);
    dpInterval = setInterval("updateDisappearingPlatforms()",100);

    // Start the game interval
    gameInterval = setInterval("gamePlay()", GAME_INTERVAL);
	  playSound("backgroundMusic");
}
function randomXY(){
  x = Math.floor(Math.random() * (SCREEN_SIZE.w - MONSTER_SIZE.w + 1));
  y = Math.floor(Math.random() * (SCREEN_SIZE.h - MONSTER_SIZE.h + 1));
  return new Point(x,y);
}
function createGoodThing(x,y){
  var goodThingEle = svgdoc.createElementNS("http://www.w3.org/2000/svg", "use");
  goodThingEle.setAttribute("x", x);
  goodThingEle.setAttribute("y", y);
  goodThingEle.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", "#goodThing");
  monsterEle = svgdoc.getElementById("goodThings").appendChild(goodThingEle);
}

//
// This function sets the zoom level to 2
//
function setZoom() {
    zoom = 2.0;
}
//
// This function removes all/certain nodes under a group
//
function cleanUpGroup(id, textOnly) {
    var node, next;
    var group = svgdoc.getElementById(id);
    node = group.firstChild;
    while (node != null) {
        next = node.nextSibling;
        if (!textOnly || node.nodeType == 3) // A text node
            group.removeChild(node);
        node = next;
    }
}


//
// This function creates the monsters in the game
//
function createMonster(id, x, y, monsterType) {
    //create monster's svg element
    var monsterEle = svgdoc.createElementNS("http://www.w3.org/2000/svg", "use");
    monsterEle.setAttribute("mydata:id", id);
    monsterEle.setAttribute("x", x);
    monsterEle.setAttribute("y", y);
    monsterEle.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", "#monster");
    monsterEle = svgdoc.getElementById("monsters").appendChild(monsterEle);
    //create monster object

	  var dest = randomXY();
    var monsterINT = new Monster(monsterEle,x,y,dest,monsterType);
    monsters[id]=monsterINT;
    console.log(monsters);
}

function moveMonster(){
    for(monster in monsters){
      //monster moving
      if(Math.abs(monsters[monster].destination.x - monsters[monster].position.x)<MONSTER_SPEED ){
          monsters[monster].position.x = monsters[monster].destination.x;
          if(Math.abs(monsters[monster].destination.y - monsters[monster].position.y)<MONSTER_SPEED){
            //reassign destination
            var dest = randomXY();
            monsters[monster].destination = dest;
            continue;
          }
      }else if(monsters[monster].destination.x > monsters[monster].position.x){
          monsters[monster].face = motionType.RIGHT;
          monsters[monster].position.x+=MONSTER_SPEED;
      }else if(monsters[monster].destination.x < monsters[monster].position.x){
          monsters[monster].face = motionType.LEFT;
          monsters[monster].needFlip = true;
          monsters[monster].position.x-=MONSTER_SPEED;
      }
      if(monsters[monster].destination.y > monsters[monster].position.y){
          monsters[monster].position.y+=MONSTER_SPEED;
      }else if(monsters[monster].destination.y < monsters[monster].position.y){
          monsters[monster].position.y-=MONSTER_SPEED;
      }
      if(monsters[monster].face==motionType.RIGHT){
          monsters[monster].node.setAttribute("transform","");
      }else {
          monsters[monster].node.setAttribute("transform","translate("+(MONSTER_SIZE.w+monsters[monster].position.x)+","+monsters[monster].position.y+")"+" scale(-1, 1)"+"translate("+(-monsters[monster].position.x)+","+(-monsters[monster].position.y)+")");
      }
      monsters[monster].node.setAttribute("x", monsters[monster].position.x);
      monsters[monster].node.setAttribute("y", monsters[monster].position.y);

      //monster shooting
      if(monsters[monster].type==MONSTER_TYPE.SHOOTER && monster_canShoot==true){
        monster_canShoot = false;
        setTimeout("monster_canShoot = true", MONSTER_SHOOT_INTERVAL);

        var bullet = svgdoc.createElementNS("http://www.w3.org/2000/svg", "use");
        bullet.setAttribute("mydata:face",monsters[monster].face);
        bullet.setAttribute("x", monsters[monster].position.x + MONSTER_SIZE.w / 2 - BULLET_SIZE.w / 2);
        bullet.setAttribute("y", monsters[monster].position.y + MONSTER_SIZE.h / 2 - BULLET_SIZE.h / 2);
        bullet.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", "#bulletM");
        svgdoc.getElementById("bullets").appendChild(bullet);
      }
    }
}

//
// This function shoots a bullet from the player
//
function shootBullet() {
    if(!cheatMode&&bulletNum<=0){
      return;
    };
    // Disable shooting for a short period of time
    canShoot = false;
    setTimeout("canShoot = true", SHOOT_INTERVAL);

    // Create the bullet using the use node
    var bullet = svgdoc.createElementNS("http://www.w3.org/2000/svg", "use");
	  bullet.setAttribute("mydata:face",player.face);
    bullet.setAttribute("x", player.position.x + PLAYER_SIZE.w / 2 - BULLET_SIZE.w / 2);
    bullet.setAttribute("y", player.position.y + PLAYER_SIZE.h / 2 - BULLET_SIZE.h / 2);
    bullet.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", "#bullet");
    svgdoc.getElementById("bullets").appendChild(bullet);
	if(!cheatMode)bulletNum--;
	svgdoc.getElementById("remainingBullets").firstChild.data = bulletNum;
	playSound("shootBulletSound");
}


//
// This is the keydown handling function for the SVG document
//
function keydown(evt) {
    var keyCode = (evt.keyCode)? evt.keyCode : evt.getKeyCode();

    switch (keyCode) {
        case "A".charCodeAt(0):
            player.motion = motionType.LEFT;
			player.face = motionType.LEFT;
            break;

        case "D".charCodeAt(0):
            player.motion = motionType.RIGHT;
			player.face = motionType.RIGHT;
            break;

        case "W".charCodeAt(0):
            if (cheatMode||player.isOnPlatform()) {
                player.verticalSpeed = JUMP_SPEED;
            }
            break;

        case 32:
            if (cheatMode||canShoot) {
				shootBullet();
			}
            break;
			
		case "C".charCodeAt(0):
			cheatMode = true;
			svgdoc.getElementById("player").setAttribute("opacity","0.5");
            break;
		
		case "V".charCodeAt(0):
			cheatMode = false;
			svgdoc.getElementById("player").setAttribute("opacity","1");
            break;
    }
}


//
// This is the keyup handling function for the SVG document
//
function keyup(evt) {
    // Get the key code
    var keyCode = (evt.keyCode)? evt.keyCode : evt.getKeyCode();

    switch (keyCode) {
        case "A".charCodeAt(0):
            if (player.motion == motionType.LEFT) player.motion = motionType.NONE;
            break;

        case "D".charCodeAt(0):
            if (player.motion == motionType.RIGHT) player.motion = motionType.NONE;
            break;
    }
}

function gameOver(){
	if(!win&&cheatMode)	return;
  // Clear the game interval
  clearInterval(gameInterval);
  clearInterval(timeInterval);
  clearInterval(dpInterval);

  svgdoc.documentElement.removeEventListener("keydown", keydown, false);
  svgdoc.documentElement.removeEventListener("keyup", keyup, false);

	stopSound("backgroundMusic");
  if(win==true){
    playSound("nextLevelSound");
    score = score + rTime * 10 * zoom;
	svgdoc.getElementById("score").firstChild.data = score;
  }else{
	  playSound("playerDieSound");
  }
  
	// Get the high score table from cookies
	var table = getHighScoreTable();
	var record = new ScoreRecord(playerName, score);
	var j = table.length;
	for (var i = 0; i < table.length; i++){
		if (record.score > table[i].score){
			j = i;
			break;
		}
	}
	table.splice(j, 0, record);
	setHighScoreTable(table);
	// Show the high score table
	svgdoc.getElementById("highscoretable").setAttribute("visibility","visible");
	showHighScoreTable(table,j);
}

function monsterDie(node){
  var SVGmonsters = svgdoc.getElementById("monsters");
  var id = node.getAttribute("mydata:id");
  delete monsters[id];
  SVGmonsters.removeChild(node);
  playSound("monsterDieSound");
}
//
// This function checks collision
//
function collisionDetection() {
    // Check whether the player collides with a monster
    var monsters = svgdoc.getElementById("monsters");
    for (var i = 0; i < monsters.childNodes.length; i++) {
        var monster = monsters.childNodes.item(i);
        var x = parseInt(monster.getAttribute("x"));
        var y = parseInt(monster.getAttribute("y"));

        if (intersect(new Point(x, y), MONSTER_SIZE, player.position, PLAYER_SIZE)) {
            gameOver();
            return;
        }
    }


    // Check whether a bullet hits a monster
    var bullets = svgdoc.getElementById("bullets");
    for (var i = 0; i < bullets.childNodes.length; i++) {
        var bullet = bullets.childNodes.item(i);

        var x = parseInt(bullet.getAttribute("x"));
        var y = parseInt(bullet.getAttribute("y"));

		// Check whether the player collides with a monster bullet
		if(bullet.getAttribute("xlink:href")=="#bulletM"){
			if (intersect(new Point(x, y), BULLET_SIZE, new Point(player.position.x, player.position.y), PLAYER_SIZE)) {
				gameOver();
				return;
			}
			continue;
        }
        for (var j = 0; j < monsters.childNodes.length; j++) {
            var monster = monsters.childNodes.item(j);
            var mx = parseInt(monster.getAttribute("x"));
            var my = parseInt(monster.getAttribute("y"));

            if (intersect(new Point(x, y), BULLET_SIZE, new Point(mx, my), MONSTER_SIZE)) {
                monsterDie(monster);
                j--;
                bullets.removeChild(bullet);
                i--;

                //write some code to update the score
					score+=MONSTER_VALUE*((zoom==2.0)?3:1);
					svgdoc.getElementById("score").firstChild.data = score;
            }
        }
    }
	// eat good things
	var goodThings = svgdoc.getElementById("goodThings");
	if (goodThings.childNodes.length==0)svgdoc.getElementById("exits").setAttribute("opacity","1.0");
	for (var i = 0; i < goodThings.childNodes.length; i++) {
        var goodThing = goodThings.childNodes.item(i);

		var pos1 = player.position;
		var size1 = PLAYER_SIZE;
		var pos2 = new Point(+goodThing.getAttribute("x"), +goodThing.getAttribute("y"));
		var	size2 = GOOD_THINGS_SIZE;
		if (intersect(pos1, size1, pos2, size2)){
			goodThing.parentNode.removeChild(goodThing);
			score += zoom*50;
			svgdoc.getElementById("score").firstChild.data = score;
		}

	}

	// go to exit
	var exit = svgdoc.getElementById("exits");
	if (intersect(player.position, PLAYER_SIZE, new Point(exit.getAttribute("x"), exit.getAttribute("y")), new Size(42, 45))){
		if (svgdoc.getElementById("goodThings").childNodes.length<=0){
			//alert("Next Level!");
			win = true;
			gameOver();
		}
	}
}


//
// This function updates the position of the bullets
//
function moveBullets() {
    // Go through all bullets
    var bullets = svgdoc.getElementById("bullets");
    for (var i = 0; i < bullets.childNodes.length; i++) {
        var node = bullets.childNodes.item(i);

        // Update the position of the bullet
        var x = parseInt(node.getAttribute("x"));
        if(node.getAttribute("mydata:face") == motionType.RIGHT){
			node.setAttribute("x", x + BULLET_SPEED);
		}else{
			node.setAttribute("x", x - BULLET_SPEED);
		}

        // If the bullet is not inside the screen delete it from the group
        if (x > SCREEN_SIZE.w|| x < 0) {
            bullets.removeChild(node);
            i--;
        }
    }
}
function createDisappearingPlatforms(){
	var platforms = svgdoc.getElementById("platforms");
  var ode = svgdoc.getElementsByClassName("dp");
  if (ode.length>0){
    for(var i = 0; i < ode.length; i++){
      ode[i].parentNode.removeChild(ode[i]);
    }
  }
	var dPlatform1 = svgdoc.createElementNS("http://www.w3.org/2000/svg", "rect");
	dPlatform1.setAttribute("class", "dp");
	dPlatform1.setAttribute("x", 1);
	dPlatform1.setAttribute("y", 239);
	dPlatform1.setAttribute("width", 80);
	dPlatform1.setAttribute("height", 20);
  dPlatform1.setAttribute("fill", "none");
	dPlatform1.setAttribute("mydata:id", "drop1");
  platforms.appendChild(dPlatform1);

  var dPlatform2 = svgdoc.createElementNS("http://www.w3.org/2000/svg", "rect");
	dPlatform2.setAttribute("class", "dp");
	dPlatform2.setAttribute("x", 209.5);
	dPlatform2.setAttribute("y", 309);
	dPlatform2.setAttribute("width", 80);
	dPlatform2.setAttribute("height", 20);
	dPlatform2.setAttribute("fill", "none");
  dPlatform2.setAttribute("mydata:id", "drop2");
  platforms.appendChild(dPlatform2);

  var dPlatform3 = svgdoc.createElementNS("http://www.w3.org/2000/svg", "rect");
  dPlatform3.setAttribute("class", "dp");
  dPlatform3.setAttribute("x", 359.491);
  dPlatform3.setAttribute("y", 449);
  dPlatform3.setAttribute("width", 80);
  dPlatform3.setAttribute("height", 20);
  dPlatform3.setAttribute("fill", "none");
  dPlatform3.setAttribute("mydata:id", "drop3");
  platforms.appendChild(dPlatform3);

  var dPlatform4 = svgdoc.createElementNS("http://www.w3.org/2000/svg", "rect");
  dPlatform4.setAttribute("class", "dp");
  dPlatform4.setAttribute("x", 440.25);
  dPlatform4.setAttribute("y", 88);
  dPlatform4.setAttribute("width", 80);
  dPlatform4.setAttribute("height", 20);
  dPlatform4.setAttribute("fill", "none");
  dPlatform4.setAttribute("mydata:id", "drop4");
  platforms.appendChild(dPlatform4);
  for (var i = 1; i < 5; i++) {
    svgdoc.getElementById("drop"+i).setAttribute("opacity", "100");
  }
}

function updateDisappearingPlatforms(){
	var dps = svgdoc.getElementsByClassName("dp");
	for(var i = 0; i < dps.length; i++){
		var x = parseInt(dps[i].getAttribute("x"));
    var y = parseInt(dps[i].getAttribute("y"));
		var dpIntArea = new Size(dps[i].getAttribute("width"), 1);
		var node = svgdoc.getElementById(dps[i].getAttribute("mydata:id"));
		if (intersect(new Point(x,y-1), dpIntArea, player.position, PLAYER_SIZE)){
			// reduce opacity
			if (+node.getAttribute("opacity") > 0.0){
				node.style.setProperty ("fill-opacity", (+node.getAttribute("opacity")-10.0)/100, null);
				node.setAttribute("opacity", (+node.getAttribute("opacity")-10.0));
			} else {
				// node.parentNode.removeChild(node);
				dps[i].parentNode.removeChild(dps[i]);
			}
		} else if (+node.getAttribute("opacity") > 0.0){
			node.style.setProperty ("fill-opacity", 1.0, null);
			node.setAttribute("opacity", 100.0);
		}
	}
}

function linkPortals(){
	var pt1 = svgdoc.getElementById("portal1");
	var pt2 = svgdoc.getElementById("portal2");

	// update player pos if touch portal
	if (intersect(player.position, PLAYER_SIZE, new Point(pt1.getAttribute("x"),pt1.getAttribute("y")), PORTAL_SIZE)){
		var newPos1 = new Point();
		newPos1.x = parseInt(pt2.getAttribute("x")) + PORTAL_SIZE.w + 1;
		newPos1.y = parseInt(pt2.getAttribute("y")) + PORTAL_SIZE.h  - PLAYER_SIZE.h;
		player.position = newPos1;
	}
	if (intersect(player.position, PLAYER_SIZE, new Point(pt2.getAttribute("x"),pt2.getAttribute("y")), PORTAL_SIZE)){
		var newPos2 = new Point();
		newPos2.x = pt1.getAttribute("x") - PORTAL_SIZE.w - 1;
		newPos2.y = pt1.getAttribute("y")  - PLAYER_SIZE.h;
		player.position = newPos2;
	}
}

//
// This function updates the position and motion of the player in the system
//
function gamePlay() {
    // Check collisions
    collisionDetection();
	linkPortals();
    // Check whether the player is on a platform
    var isOnPlatform = player.isOnPlatform();

    // Update player position
    var displacement = new Point();

    // Move left or right
    if (player.motion == motionType.LEFT)
        displacement.x = -MOVE_DISPLACEMENT;
    if (player.motion == motionType.RIGHT)
        displacement.x = MOVE_DISPLACEMENT;

    // Fall
    if (!isOnPlatform && player.verticalSpeed <= 0) {
        displacement.y = -player.verticalSpeed;
        player.verticalSpeed -= VERTICAL_DISPLACEMENT;
    }

    // Jump
    if (player.verticalSpeed > 0) {
        displacement.y = -player.verticalSpeed;
        player.verticalSpeed -= VERTICAL_DISPLACEMENT;
        if (player.verticalSpeed <= 0)
            player.verticalSpeed = 0;
    }

    // Get the new position of the player
    var position = new Point();
    position.x = player.position.x + displacement.x;
    position.y = player.position.y + displacement.y;

    // Check collision with platforms and screen
    player.collidePlatform(position);
    player.collideScreen(position);

    // Set the location back to the player object (before update the screen)
    player.position = position;

    // Move the bullets
    moveBullets();

    updateScreen();
}


//
// This function updates the position of the player's SVG object and
// set the appropriate translation of the game screen relative to the
// the position of the player
//
function updateScreen() {
    // Transform the player
	if(player.face==motionType.RIGHT){
		player.node.setAttribute("transform","translate(" + (player.position.x + PLAYER_SIZE.w) + ", "+player.position.y+") scale(-1, 1)");
	}else{
		player.node.setAttribute("transform", "translate(" + player.position.x + "," + player.position.y + ")");
	}
	var playerNameNode = svgdoc.getElementById("playerName");
	playerNameNode.setAttribute("x", player.position.x+20);
    playerNameNode.setAttribute("y", player.position.y-10);

	moveMonster();

    // Calculate the scaling and translation factors
    var scale = new Point(zoom, zoom);
    var translate = new Point();

    translate.x = SCREEN_SIZE.w / 2.0 - (player.position.x + PLAYER_SIZE.w / 2) * scale.x;
    if (translate.x > 0)
        translate.x = 0;
    else if (translate.x < SCREEN_SIZE.w - SCREEN_SIZE.w * scale.x)
        translate.x = SCREEN_SIZE.w - SCREEN_SIZE.w * scale.x;

    translate.y = SCREEN_SIZE.h / 2.0 - (player.position.y + PLAYER_SIZE.h / 2) * scale.y;
    if (translate.y > 0)
        translate.y = 0;
    else if (translate.y < SCREEN_SIZE.h - SCREEN_SIZE.h * scale.y)
        translate.y = SCREEN_SIZE.h - SCREEN_SIZE.h * scale.y;

    // Transform the game area
    svgdoc.getElementById("gamearea").setAttribute("transform", "translate(" + translate.x + "," + translate.y + ") scale(" + scale.x + "," + scale.y + ")");
}

function playSound(id){
	svgdoc.getElementById(id).pause();
	svgdoc.getElementById(id).currentTime = 0;
	svgdoc.getElementById(id).play();
}

function stopSound(id){
	svgdoc.getElementById(id).pause();
	svgdoc.getElementById(id).currentTime = 0;
}
